# -*- coding: utf-8 -*-
# -------------------------------------------------------------------------------------------------------#
# Filename: wssi_prep_functions.py                                                                       #
# Description: Winter Storm Severity Index (WSSI) - preparation functions                                #
# Authors: Nathan Foster/Mike Muccilli/Josh Kastman                                                      #
# Contact: nathan.foster@noaa.gov, michael.muccilli@noaa.gov, joshua.kastman@noaa.gov                    #
# Established Functions - Kastman 2019-03-19                                                             # 
# Change Log:                                                                                            #
# Download grib data via batch/powershell insteady of urllib - Kastman - 2021-02-22                      #
# Switched to WPC compute farm for NDFD data for data source; Files moved via SCP - Kastman - 2021-06-21 #
# Added list_master() to manage grids/time for 3/4 time periods - Kastman - 2021-07-13                   #                                                                                                       #
#--------------------------------------------------------------------------------------------------------#

#imports
import arcpy, os, traceback, subprocess, urllib2, shutil
from time import gmtime, strftime 
from datetime import datetime, timedelta
import logging
import traceback
import glob
import wssi_config as cfg
from wssi_general_functions import clean, rolling_periods

start_time = strftime("%a, %d %b %Y %H:%M:%S", gmtime())

#--------------------------------------------------------------#
# log_prep() - Archieves old log, cleans directory for new log #
#--------------------------------------------------------------#

def log_prep():
    dest_dir = cfg.log_dir+"\\log_archive"
    log_dir = cfg.log_dir+"\\data\\logs\\"
    try:
        print ("moving previous log to archive")
        for log in glob.glob(cfg.log_dir+"*.log"):
            print(log)
            shutil.move(log, dest_dir)
        #print ("log archive complete")
    except Exception as e:
        print(traceback.format_exc())

#------------------------------------------------------------------#
# Get latest NDFD from WPC Compute Farm via PSCP                   #
# Windows Batch Scripts Description:                               #
# Establish latest file uploaded on CF from NCO for each variable, #
# pass that file name as a variable to PSCP for download           #
#------------------------------------------------------------------#

def get_ndfd():
    subprocess.call(cfg.scripts+"get_ndfd.bat")
    print ('get day 4 data for wgust and temp')
    d4_grids = ['wgust']#temp
    for b in d4_grids:
        fullurl = 'ftp://tgftp.nws.noaa.gov/SL.us008001/ST.opnl/DF.gr2/DC.ndfd/AR.conus/VP.004-007/ds.'+b+'.bin'
        f = urllib2.urlopen(fullurl)
        html = f.read()
        f.close()
        fout = open(str(cfg.grb_dir)+'ds.'+b+'_d4.bin' , "wb")
        fout.write(html)
        fout.close()
    logging.debug('NDFD and WPC Data Downloaded')
    logging.error('An Error has occured during the download process')
        
#-----------------#
# End of Function #
#-----------------#

#---------------------------------------------------------#    
# degrib_txt() - Uses DeGrib program to create text files #
# containing information about grib data and to generate  #
# .flt files, projects .flts using arcpy, and establishes # 
# time variables for all WSSI processing                  #
#---------------------------------------------------------#

snow_dict= {}
d4snow_dict = {}
d4qpf_dict = {}
d4wgust_final = {}
d4wgust_dict = {}
temp_dict = {}
wgust_dict = {}
qpf_dict = {}   
ice_dict = {}

def degrib_ndfd():

    #print (cfg.grid_list)
    for grid in cfg.grid_list:
        #print (grid)
        os.chdir(cfg.degribdir)
        filename = 'ds.' + grid + '.bin'
        print filename
        logging.debug('degrib: '+filename)
        cmd = 'degrib.exe -I -in ' + str(cfg.grb_dir) + str(filename) + ' >' + str(cfg.grb_dir) + str(grid) + '.txt'
        os.system(cmd)
        file = cfg.grb_dir + str(grid) + '.txt'
        data = open(file, 'r')
        
        headerline = data.readline()
        headerline = headerline.strip()
        hl = headerline.split(',')[0]
        lines = str(data.read()).splitlines()
        #print hl
        if hl == 'MsgNum':
            pass
            #print 'No Error Lines'
        if hl[:7] == 'Warning':
            #print hl[:7]+" !!! Error Found"
            lines = lines[3:]
        for line in lines:
            if line != str(headerline):
                line = line.split(',')
                msgnum = line[0]
                msgnum = msgnum.split('.')
                msgnum = msgnum[0]
                fcstHR = line[-1]
                fcstHR = fcstHR.split('.')
                fcstHR = fcstHR[0]
                if int(fcstHR) > 96:
                    print ('fcst > 96 hours. Breaking loop')
                    logging.debug('fcst > 96 hours. Breaking loop')
                    break
                vldHR = line[-2]
                vldHR = vldHR.split('.')
                vldHR = vldHR[0]# use for just hour --> [-5:-3]
                #print (str(msgnum)+" | "+str(fcstHR)+" | "+str(vldHR))

                #--------------------------------------------------------------------------#
                # Pack msgnum, fcstHR and vldHR into a nested dict. Output is in the form: #
                # msgnum: {'fhr': fcstHR, 'valid' : vldHR}. For example                    #
                # num_dict = {1: {'valid': ' 07/15/2021 06:00', 'fhr': ' 65'}}             #
                # Where 1 is the grib message number, 'valid' is the Valid UTC time and    #
                # fhr is the proj (forecast) HR. These list are created for each .bin file #
                #--------------------------------------------------------------------------#
                
                if msgnum.isdigit():
                    if grid == 'temp':
                        temp_dict[msgnum] = {'fhr' : fcstHR, 'valid' : vldHR}
                        num_dict = temp_dict
                    if grid == 'wgust':
                        wgust_dict[msgnum] = {'fhr' : fcstHR, 'valid' : vldHR}
                        num_dict = wgust_dict
                    if grid == 'qpf':
                        qpf_dict[msgnum] = {'fhr' : fcstHR, 'valid' : vldHR}
                        num_dict = qpf_dict
                    if grid == 'iceaccum':
                        ice_dict[msgnum] = {'fhr' : fcstHR, 'valid' : vldHR}
                        num_dict = ice_dict
                    if grid == 'snow':
                        snow_dict[msgnum] = {'fhr' : fcstHR, 'valid' : vldHR}
                        num_dict = snow_dict
                    if grid == 'wpc_d4snow':
                        d4snow_dict[msgnum] = {'fhr' : fcstHR, 'valid' : vldHR}
                        num_dict = d4snow_dict
                    if grid == 'wpc_d4qpf':
                        d4qpf_dict[msgnum] = {'fhr' : fcstHR, 'valid' : vldHR}
                        num_dict = d4qpf_dict
                    #if grid == 'temp_d4':
                    #    cfg.d4temp_dict[msgnum] = {'fhr' : fcstHR, 'valid' : vldHR}
                    #    num_dict = d4temp_dict
                    if grid == 'wgust_d4':
                        d4wgust_dict[msgnum] = {'fhr' : fcstHR, 'valid' : vldHR}
                        num_dict = d4wgust_dict
        


       #print grid+" num_dict ="+str(num_dict)
        for n in num_dict.keys():
            #print n
            namestyle = ' -nameStyle ' + '"' + str(grid) + str(n) + '.txt"'
            cmd = 'degrib.exe -in ' + str(cfg.grb_dir) + str(filename) + ' -C -msg ' + str(n) + ' -Met -Flt -namePath ' + str(cfg.grb_dir) + str(namestyle)
            os.system(cmd)
            # Process: Define Projection
            ndfd_grids = str(cfg.grb_dir) + str(grid) + str(n) + '.flt'
            arcpy.DefineProjection_management(ndfd_grids, "PROJCS['NDFD',GEOGCS['GCS_Unknown',DATUM['D_unknown',SPHEROID['Sphere',6371000.0,0.0]],PRIMEM['Greenwich',0.0],UNIT['Degree',0.0174532925199433]],PROJECTION['Lambert_Conformal_Conic'],PARAMETER['false_easting',0.0],PARAMETER['false_northing',0.0],PARAMETER['central_meridian',-95.0],PARAMETER['standard_parallel_1',25.0],PARAMETER['standard_parallel_2',25.0],PARAMETER['latitude_of_origin',25.0],UNIT['Meter',1.0]]")

    #-----------------------------------------------------------------------------------------#
    # list_master() creates master lists to sync up day 3/4 timing between NDFD and WPC data. #
    #-----------------------------------------------------------------------------------------# 
    precip_num_list = []
    for s in snow_dict.keys():
        precip_num_list.append(int(s))

    precip_num_list.sort()
    #print precip_num_list

    precip_hrs=[]
    for n in precip_num_list:
        precip_hrs.append(cfg.rdict[str(n)])
    #print "precip_hrs list from prep: "+str(precip_hrs)
    
    #cfg.precip_num_list = precip_num_list
    #cfg.precip_hrs = precip_hrs

    fout = open(cfg.work_dir+'precp_num_list.txt' , "wb")
    for n in precip_num_list:
        fout.write(str(n)+' ')
    fout.close()

    f = open(cfg.work_dir+'precip_hrs.txt' , "wb")
    for n in precip_hrs:
        f.write(str(n)+' ')    
    f.close()

    logging.debug('degrib functions finished')

#-----------------#
# End of Function #
#-----------------#

#-----------------------------------------------------------------------------------------#
# list_master() creates master lists to sync up day 3/4 timing between NDFD and WPC data. #
#-----------------------------------------------------------------------------------------#

def list_master():
    def valid_max(var):
        numlist = []
        for n in var.keys():
            #print n
            numlist.append(int(n))
        #print numlist
        max_var = str(max(numlist))
        #print max_var
        max_vld = var[str(max_var)]['valid']
        #max_fhr = var[str(max_var)]['fhr']
        return max_var,max_vld #,max_fhr
     
    #print valid_max(var = cfg.snow_dict)
    sn_max_vld = valid_max(var = snow_dict)[1][1:]
    sn_max_grd = valid_max(var = snow_dict)[0]
    #print 'sn max vld: '+sn_max_vld
    #print 'sn max grf: '+sn_max_grd

    wgust_max_vld = valid_max(var = wgust_dict)[1][1:]
    wgust_max_grd = valid_max(var = wgust_dict)[0]
    #print 'wgust max vld: '+wgust_max_vld
    #print 'wgust max grf: '+wgust_max_grd
    
    

    #--------------------------------------------------------------------------------#
    # Convert valid dates to time objects, find the difference in hours to determine #
    # how many day 4 grids are needed to match the end of day 3.                     #
    #--------------------------------------------------------------------------------#

    # Convert last valid dates to time object 
    
    format = '%m/%d/%Y %H:%M'
    sn_DT_str = datetime.strptime(sn_max_vld, format)
    wgust_DT_str = datetime.strptime(wgust_max_vld, format)
    #print sn_DT_str
    #print wgust_DT_str
    # compute the difference between snow and wind/temp, convert to hours
    sn_m_wgust = sn_DT_str - wgust_DT_str
    sn_m_wgust_dis = sn_m_wgust.total_seconds()
    sn_m_wgust_hr_dif = divmod(sn_m_wgust_dis, 3600)[0]
    #print sn_m_wgust_hr_dif
    #----------------------------------------------------------------------------------#
    # divide hour difference by 6 to determine the number of d4 grids needed from NDFD #
    # 6 is used because it is the timestep in NDFD beyon day 3                         # 
    #----------------------------------------------------------------------------------#
    d4_grids = sn_m_wgust_hr_dif / 6
    d4_grids = int(d4_grids)
    #print "d4grids: "+str(d4_grids)

    w = open(cfg.work_dir+'wgust_max_valid.txt' , "wb")
    w.write(str(wgust_max_grd))
    w.close()

    g = 0
    for g in range(0, int(d4_grids)):
        g = g+1
        msgnum = int(wgust_max_grd) + g
        d4_entry = d4wgust_dict[str(g)]
        fcstHR = d4_entry['fhr']
        vldHR = d4_entry['valid']
        #print "add a new msgnum to list: "+str(msgnum)
        wgust_dict[str(msgnum)] = {'fhr' : fcstHR, 'valid' : vldHR}

    #print wgust_dict
    #wdict = open(cfg.work_dir+'wgust_dict.txt' , "wb")
    #wdict.write(str(wgust_dict))
    #wdict.close()


    #------------------------------#
    # Establish wgust global lists #
    #------------------------------#
    wgust_num_list = []
    for w in wgust_dict.keys():
        wgust_num_list.append(int(w))
    wgust_num_list.sort()
    wgust_hrs = []
    for w in wgust_num_list:  
        wgust_hrs.append(wgust_dict[str(w)]['fhr'][1:])

    #print wgust_num_list
    #print wgust_hrs


    wgnum = open(cfg.work_dir+'wgust_num_list.txt' , "wb")
    for n in wgust_num_list:
        wgnum.write(str(n)+' ')
    wgnum.close()

    wghr = open(cfg.work_dir+'wgust_hrs.txt' , "wb")
    for n in wgust_hrs:
        wghr.write(str(n)+' ')
    wghr.close()


    d4wgust_num_list = []
    for w in d4wgust_dict.keys():
        d4wgust_num_list.append(int(w))
    d4wgust_num_list.sort()

    d4wgnum = open(cfg.work_dir+'d4wgust_num_list.txt' , "wb")
    for n in d4wgust_num_list:
        d4wgnum.write(str(n)+' ')
    d4wgnum.close()        

    d4wgust_hrs = []
    for w in d4wgust_num_list:  
        d4wgust_hrs.append(d4wgust_dict[str(w)]['fhr'][1:])

    d4wghr = open(cfg.work_dir+'d4wgust_hrs.txt' , "wb")
    for n in d4wgust_hrs:
        d4wghr.write(str(n)+' ')
    d4wghr.close()
##    cfg.wgust_num_list = wgust_num_list
##    cfg.wgust_hrs = wgust_hrs
##    cfg.wgust_dict = wgust_dict
##    cfg.d4wgust_dict = d4wgust_dict


    #-----------------------------------------------------------#
    # Find last wgust hour to determine beginning of day 4 list #
    #-----------------------------------------------------------#
    wgust_check_list = []
    for w in wgust_dict.keys():
        wgust_check_list.append(int(w)) 

    wgust_check_list.sort()
    #print (wgust_check_list)
    last_gust = wgust_check_list[-1]
    #print "last_gust "+str(last_gust)
    last_wgustHR = wgust_dict[str(last_gust)]['fhr'][1:]
    #print ("last_wgustHR "+str(last_wgustHR))

    x = 0
    if int(last_wgustHR) == 60:
        g = g + 2
        x = x + 2
    if int(last_wgustHR) == 66:
        g = g + 1
        x = x + 1
    else:
        g = g + 0
        x = x + 0
    
    #print ('val of g: '+str(g))
    
    maxg = g + 4
    #print "maxg: "+str(maxg)
    
    for g in range(g, int(maxg)):
        g = g+1
        x = x + 1
        msgnum = x 
        d4_entry = d4wgust_dict[str(g)]
        fcstHR = d4_entry['fhr']
        vldHR = d4_entry['valid']
        #print "add a new msgnum to list: "+str(msgnum)
        #print "fcstHR: "+fcstHR
        d4wgust_final[str(msgnum)] = {'fhr' : fcstHR, 'valid' : vldHR}

    #print d4wgust_final
    #cfg.d4wgust_final = d4wgust_final
    logging.debug('list master finished')
    return wgust_max_grd, wgust_max_vld



def date_time_mgt():

    valid = snow_dict['4']['valid'][1:]
    print valid
    ds_24 = datetime.strptime(str(valid), '%m/%d/%Y %H:%M')
    
    base = cfg.fcst_hr_list
    print base
    delta_hrs = base[:-4]
    delta_hrs.insert(0,"0")
    print delta_hrs
    r24_hrs = base[3:]
    print r24_hrs
    for d,r in zip (delta_hrs,r24_hrs):
        fcstTD = ds_24 + timedelta(hours=int(d))
        fcstHR = fcstTD.strftime("%HZ %a %d %b, %Y")
        print fcstHR
        out=open(cfg.xmls+"fhr_"+str(r)+".txt", 'wb')
        out.write(fcstHR)
        out.close()
        
    # Forecast Initial:
    fcstTD = ds_24 + timedelta(hours=-24)
    fcst_init = fcstTD.strftime("%HZ %a %d %b, %Y")
    out=open(cfg.xmls+"init.txt", 'wb')
    out.write(fcst_init)
    out.close()        

# execute #
log_prep()
# set up log file
log_stamp = strftime("%Y%m%d%H", gmtime())
logging.basicConfig(filename=cfg.log_dir+'/wssi_log_'+str(log_stamp)+'_prep.log',level=logging.DEBUG, format='%(asctime)s %(message)s', datefmt='%m/%d/%Y %I:%M:%S %p')

#print ('clean dirs')
clean(cfg.grb_dir)
clean(cfg.work_dir)
clean(cfg.roll_out)
#print ('det latest NDFD data')
get_ndfd()
print ('generate .flts')
degrib_ndfd()
list_master()
date_time_mgt()
finish_time = strftime("%a, %d %b %Y %H:%M:%S", gmtime())
logging.debug ("WSSI Preperation Scripts Start Time "+start_time)
logging.debug ("WSSI Preperation Scripts Finished Time "+finish_time)
logging.shutdown()

