# -*- coding: utf-8 -*-
#--------------------------------------------------------------------#
# Snow Load Index                                                    #
# Filename: snow_load.py                                             #
# Description: Generates snow rates and 'leaf off' snow load amounts #
# Authors: Nathan Foster/Mike Muccilli/Andy Nash/Josh Kastman        #         #
# Change Log:                                                        #
# modified categories 12/1/16 AN to bring thresholds down a bit      #
# Established Functions - Kastman 2019-03-19                         #
# Updated for rolling 24 hr output - Kastman 2021-07-02              # # 
#--------------------------------------------------------------------#


#import arcview, arcpy, os, shutil, ftplib, datetime, time, traceback, urllib2
import arcpy, os, shutil, ftplib, datetime, time, traceback, urllib2
import traceback
import logging
from arcpy import env
from time import gmtime, strftime
from shutil import copyfile
import glob
from arcpy.sa import *
from datetime import datetime

start_time = strftime("%a, %d %b %Y %H:%M:%S", gmtime())
import wssi_config as cfg
import wssi_general_functions as gfunc

#from datetime import datetime
log_stamp = strftime("%Y%m%d%H", gmtime())
logging.basicConfig(filename=cfg.log_dir+'/wssi_log_'+str(log_stamp)+'_SnLdWin.log',level=logging.DEBUG, format='%(asctime)s %(message)s', datefmt='%m/%d/%Y %I:%M:%S %p')


#def sn_ld_winter(**time_var_kwargs):
def sn_ld_winter():
    
    #-----------------#
    # Set local lists #
    #-----------------#
    file = cfg.work_dir+'precp_num_list.txt'
    data = open(file, 'r')
    precip_num_text = data.readline()
    precip_num_list = precip_num_text.split()
    
    file = cfg.work_dir+'precip_hrs.txt'
    data = open(file, 'r')
    precip_hrs_txt = data.readline()
    precip_hrs = precip_hrs_txt.split()

    precip_num_list_d4 = cfg.precip_num_list_d4
    precip_d4_hrs = cfg.precip_d4_hrs
    rdict = cfg.rdict
    rdict_d4 = cfg.rdict_d4

    all_hrs = []
    for hr in precip_hrs:
        all_hrs.append(hr)
    for d4hr in precip_d4_hrs:
        all_hrs.append(d4hr)
    
    rlist_6hr = gfunc.rolling_periods(fp = precip_hrs[-1], rHR=4, step=6, hr_list = precip_hrs)
    rlist_nums = gfunc.rolling_periods(fp = precip_num_list[-1], rHR=4, step=6, hr_list = precip_num_list)


    
    #Get Spatial Analyst License
    try:
        if arcpy.CheckExtension("Spatial") == "Available":
            arcpy.CheckOutExtension("Spatial")
        else:
            # Raise a custom exception
            #
            raise LicenseError

    except LicenseError:
        print "Spatial Analyst license is unavailable"  

    arcpy.env.overwriteOutput = True
    arcpy.SetLogHistory(False)

    print 'Starting Snow Load Winter Index'
    logging.debug ('Starting Snow Load Winter Index')

    
    ##############
    # DAYs 1 - 3 #
    ##############
    
    swe_list = []    
    for n in precip_num_list:
        try:
            arcpy.gp.Con_sa(cfg.grb_dir+"snow"+str(n)+".flt", cfg.grb_dir+"qpf"+str(n)+".flt", cfg.work_dir+"SWEg2_"+rdict[str(n)]+".tif", "0", "\"VALUE\" > 2")
            swe_list.append(cfg.work_dir+"SWEg2_"+rdict[str(n)]+".tif")
        except Exception as e:
            print(traceback.format_exc())
            logging.error(traceback.format_exc())
            
    #########
    # DAY 4 #
    #########    

    swe_d4_list = []    
    for n in precip_num_list_d4:
        try:
            arcpy.gp.Con_sa(cfg.grb_dir+"wpc_d4snow"+str(n)+".flt", cfg.grb_dir+"wpc_d4qpf"+str(n)+".flt", cfg.work_dir+"SWEg2_"+rdict_d4[str(n)]+".tif", "0", "\"VALUE\" > 2")
            swe_d4_list.append(cfg.work_dir+"SWEg2_"+rdict_d4[str(n)]+".tif")
        except Exception as e:
            print(traceback.format_exc())
            logging.error(traceback.format_exc())


    # Set Mask environment
    arcpy.env.mask = cfg.States
    # Calculate total SWE

    ##############
    # DAYs 1 - 3 #
    ##############
    
    # total SWE
    swe_join = ';'.join(swe_list)
    #print swe_join
    arcpy.gp.CellStatistics_sa(swe_join, cfg.work_dir+"Total_SWE.tif", "SUM", "DATA")
    arcpy.gp.Times_sa(cfg.work_dir+"Total_SWE.tif", cfg.evergreen, cfg.work_dir+"LandUse_SWE.tif")
    arcpy.gp.Reclassify_sa(cfg.work_dir+"LandUse_SWE.tif", "Value", "0 0.3 0;0.3 0.75 2;0.75 1.0 4;1.0 1.5 6;1.5 2.0 8;2.0 10 10", cfg.work_dir+"WSII_SnowLoad.tif", "DATA")
         
    #########
    # DAY 4 #
    #########

    # total SWE
    swe_d4_join = ';'.join(swe_d4_list)
    #print swe_d4_join
    arcpy.gp.CellStatistics_sa(swe_d4_join, cfg.work_dir+"Total_thru_d4_SWE.tif", "SUM", "DATA")
    arcpy.gp.Times_sa(cfg.work_dir+"Total_thru_d4_SWE.tif", cfg.evergreen, cfg.work_dir+"LandUse_d4_SWE.tif")
    arcpy.gp.Reclassify_sa(cfg.work_dir+"LandUse_d4_SWE.tif", "Value", "0 0.3 0;0.3 0.75 2;0.75 1.0 4;1.0 1.5 6;1.5 2.0 8;2.0 10 10", cfg.work_dir+"WSII_thru_d4_SnowLoad.tif", "DATA")

    ##############
    # DAYs 1 - 3 #
    ##############

    # Rolling SWE #
    for r in range (0,len(rlist_nums)):
        ravg = []
        for s in range (0,4):
            ravg.append(cfg.work_dir+"SWEg2_"+rdict[str(rlist_nums[r][s])]+".tif")
        ravg_join = ';'.join(ravg)
        print (ravg_join)
        arcpy.gp.CellStatistics_sa(ravg_join, cfg.work_dir+"f"+rdict[str(rlist_nums[r][s])]+"_SWE.tif", "SUM", "DATA")
        arcpy.gp.Times_sa(cfg.work_dir+"f"+rdict[str(rlist_nums[r][s])]+"_SWE.tif", cfg.evergreen, cfg.work_dir+"f"+rdict[str(rlist_nums[r][s])]+"_LandUse_SWE.tif")
        arcpy.gp.Reclassify_sa(cfg.work_dir+"f"+rdict[str(rlist_nums[r][s])]+"_LandUse_SWE.tif", "Value", "0 0.3 0;0.3 0.75 2;0.75 1.0 4;1.0 1.5 6;1.5 2.0 8;2.0 10 10", cfg.work_dir+"WSSI_"+rdict[str(rlist_nums[r][-1])]+"_SnLd.tif", "DATA")
        #print ('rolling 24 hr snow generated: snow_r24h_f'+rdict[rlist_precip[r][-1]]+'.tif')
        #logging.debug ('rolling 24 hr snow generated: snow_r24h_f'+rdict[rlist_precip[r][-1]]+'.tif')

    #########
    # DAY 4 #
    #########

    d4 = cfg.work_dir+"SWEg2_78.tif;"+cfg.work_dir+"SWEg2_84.tif;"+cfg.work_dir+"SWEg2_90.tif;"+cfg.work_dir+"SWEg2_96.tif;"
    arcpy.gp.CellStatistics_sa(d4, cfg.work_dir+"f96_SWE.tif", "SUM", "DATA")
    arcpy.gp.Times_sa(cfg.work_dir+"f96_SWE.tif", cfg.evergreen, cfg.work_dir+"f96_LandUse_SWE.tif")
    arcpy.gp.Reclassify_sa(cfg.work_dir+"f96_LandUse_SWE.tif", "Value", "0 0.3 0;0.3 0.75 2;0.75 1.0 4;1.0 1.5 6;1.5 2.0 8;2.0 10 10", cfg.work_dir+"WSSI_96_SnLd.tif", "DATA")

    
    arcpy.Delete_management(cfg.in_mem)
    logging.debug ('Snow Load Winter Finished')


    #----------------------------------------------------------------------------------------------------#
    # Copy total, day1 and day2 output to components output folder. Day3 will be coppied in summary step #
    #----------------------------------------------------------------------------------------------------#
    os.chdir(cfg.work_dir)
    if precip_num_list[-1] == str(11):
        d3 = "WSSI_66_SnLd.tif"
    else:
        d3 = "WSSI_66_SnLd.tif"
    infiles = ["WSII_SnowLoad.tif","WSII_thru_d4_SnowLoad.tif","WSSI_24_SnLd.tif","WSSI_48_SnLd.tif","WSSI_96_SnLd.tif",d3]
    outfiles = ["sn_ld_win_fp.tif","sn_ld_win_d4fp.tif","sn_ld_win_d1.tif","sn_ld_win_d2.tif","sn_ld_win_d4.tif","sn_ld_win_d3.tif"]
    for i,o in zip (infiles,outfiles):
        copyfile(cfg.work_dir+i, cfg.comp_out+o)


    rolling_web_list = []
    for file in glob.glob("*_SnLd.tif"):
        rolling_web_list.append(file)
    print rolling_web_list

    for f in rolling_web_list:
        rf = 'r24h_SnowLoadWin_f'+f[5:7]+'.tif'
        copyfile(cfg.work_dir+f, cfg.roll_out+rf)        

    end_time = strftime("%a, %d %b %Y %H:%M:%S", gmtime())    
    logging.debug ("Ice Accumulation Processing Start Time "+start_time)
    logging.debug ("Ice Accumulation Processing Script Finished "+end_time)

###########################################################################################################
# Execute Function#
sn_ld_winter()
logging.shutdown()
